package org.jetbrains.intellij.sharedIndexes.local

import java.nio.file.Path
import java.nio.file.Paths


interface Args {
  val indexesDir: Path
  fun downloadUrl(key: String): String
}

class ArgsImpl(args: Array<String>) : Args {
  private fun arg(args: Array<String>, key: String, default: String? = null): String {
    val keyValue = args.find { it.startsWith("--$key=") }
    if (keyValue == null) {
      requireNotNull(default) { "Required option '--$key=' is not specified " }
      return default
    }
    return keyValue.substringAfter("--$key=").trim()
  }

  override val indexesDir: Path = arg(args, "indexes-dir").let { Paths.get(it) }
  private val baseUrl = arg(args, "url").trim().removeSuffix("/").trim()

  override fun downloadUrl(key: String): String = listOf(baseUrl, key)
    .map { it.trim('/') }
    .filter { it.isNotBlank() }
    .joinToString("/")

  companion object {
    fun usage() {
      println("Shared Indexes List Generator")
      println("Usage:")
      println(" [tool] \\")
      println("    --indexes-dir=<dir of the indexes> \\")
      println("    --url=<base URL>")
      println()
    }
  }
}
