package org.jetbrains.intellij.sharedIndexes.local

import com.intellij.indexing.shared.cdn.*
import com.intellij.indexing.shared.cdn.CdnSnapshotLocal.addFiles
import com.intellij.indexing.shared.cdn.CdnSnapshotLocal.loadLocalDiskContent
import com.intellij.indexing.shared.cdn.CdnSnapshotLocal.localCdnLocalEntries
import com.intellij.indexing.shared.cdn.CdnSnapshotLocal.wipeFiles
import kotlin.system.exitProcess

fun main(argsArray: Array<String>) {
  println("Running the 'cdn-layout-tool' tool $toolVersion")
  val args = try {
    ArgsImpl(argsArray)
  }
  catch (e: Exception) {
    println("Failed to parse command line options: ${e.message}")
    ArgsImpl.usage()
    return
  }

  try {
    args.main()
  }
  catch (t: Throwable) {
    println("Unexpected failure. ${t.message}")
    t.printStackTrace()
    exitProcess(1)
  }
}

private val toolVersion: String by lazy {
  Args::class.java.getResource("/META-INF/cdn-layout-tool-version.txt")?.readText() ?: "dev"
}

fun Args.main() {
  val context = object : CdnUploadContext{
    override fun downloadUrl(entry: CdnEntry): String = this@main.downloadUrl(entry.key)

    override fun <E, R> loadContent(entries: Iterable<E>, entry: E.() -> CdnEntry, processor: (E, ByteArray) -> R): Collection<R> {
      return loadLocalDiskContent(entries, entry, processor)
    }
  }

  val masterPlan = rebuildCdnLayout(context, listOf(), localCdnLocalEntries(indexesDir))
  wipeFiles(indexesDir, masterPlan.removeEntries)
  addFiles(indexesDir, masterPlan.newEntries)
}
